package dw.samples.ldapcontrols;

import javax.naming.ldap.Control;

/**
 * A Java control class to employ the Password Policy Control.
 * 
 * Its objects are used as request controls, furthermore the class acts as
 * superclass for PasswordPolicyResponseControl.
 * 
 * developerWorks, Take advantage of IBM Tivoli Directory Server's LDAP Controls
 * via Java and JNDI.
 * 
 * @author Stefan Zoerner
 */
public class PasswordPolicyControl implements Control {

    /** OID of the Password Policy Control */
    public static final String OID = "1.3.6.1.4.1.42.2.27.8.5.1";

    private boolean critical;

    /**
     * Creates a non-critical (request) control.
     */
    public PasswordPolicyControl() {
        this(Control.NONCRITICAL);
    }

    /**
     * Creates a (request) control.
     * 
     * @param critical
     *            indicates whether control is critical for the client
     */
    public PasswordPolicyControl(boolean critical) {
        setCritical(critical);
    }

    /**
     * Returns the OID of the Password Policy Control.
     */
    public String getID() {
        return OID;
    }

    /**
     * Returns whether the control is critical for the client.
     */
    public boolean isCritical() {
        return critical;
    }

    /**
     * Sets the critical property.
     * 
     * @param critical
     *            indicates whether control is critical for the client
     */
    public void setCritical(boolean critical) {
        this.critical = critical;
    }

    /**
     * Retrieves the ASN.1 BER encoded value of the LDAP control. Here the value
     * is always absent.
     * 
     * @return always null
     */
    public byte[] getEncodedValue() {
        return null;
    }
}