package dw.samples.ldapcontrols;

import java.io.ByteArrayInputStream;
import java.io.IOException;

import javax.naming.ldap.Control;
import javax.naming.ldap.ControlFactory;

import netscape.ldap.ber.stream.BERElement;
import netscape.ldap.ber.stream.BEREnumerated;
import netscape.ldap.ber.stream.BERInteger;
import netscape.ldap.ber.stream.BERSequence;
import netscape.ldap.ber.stream.BERTag;

/**
 * Transforms a control object to a PasswordPolicyResponseControl object, if
 * appropriate. Note that Netscape Directory SDK for Java is necessary to
 * compile and run the example.
 * 
 * developerWorks, Take advantage of IBM Tivoli Directory Server's LDAP Controls
 * via Java and JNDI. Listing 10: Fragment of class
 * PasswordPolicyControlFactory.
 * 
 * @author Stefan Zoerner
 */
public class PasswordPolicyControlFactory extends ControlFactory {

    /**
     * Creates an instance of PasswordPolicyResponseControl if the passed
     * control is a response control of this type. Attributes of the result are
     * filled with the correct values (e.g. error code).
     * 
     * @param ctl
     *            the control the check
     * @return a response control of type PasswordPolicyResponseControl, or null
     */
    public Control getControlInstance(Control ctl) {
        Control result = null;

        if (ctl.getID().equals(PasswordPolicyControl.OID)) {
            try {
                PasswordPolicyResponseControl rctl = new PasswordPolicyResponseControl();
                if (ctl.getEncodedValue() != null) {
                    rctl.setEncodedValue(ctl.getEncodedValue());
                    int[] bread = { 0 };
                    BERSequence seq = (BERSequence) BERElement.getElement(
                            new SpecificTagDecoder(), new ByteArrayInputStream(ctl
                                    .getEncodedValue()), bread);
                    for (int i = 0; i < seq.size(); i++) {
                        handleSequenceElement(seq.elementAt(i), rctl);
                    }
                }
                result = rctl;
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return result;
    }

    /**
     * Analyses an element in the response sequence, either an error or a
     * warning. It transferres the corresponding values (error code, timeout) to
     * the control object target.
     * 
     * @param element
     *            BER element to handle by te method
     * @param target
     *            target response control object
     */
    protected void handleSequenceElement(BERElement element,
            PasswordPolicyResponseControl target) {

        BERTag tag = (BERTag) element;

        // warning
        if ((tag.getTag() ^ BERTag.CONTEXT) == 0) {
            BERSequence sequence = (BERSequence) tag.getValue();
            BERTag elem = (BERTag) sequence.elementAt(0);
            sequence = (BERSequence) elem.getValue();
            BERInteger intValue = (BERInteger) sequence.elementAt(0);
            if ((elem.getTag() ^ BERTag.CONTEXT) == 0) {
                target.setTimeBeforeExpiration(intValue.getValue());
            }
            if ((elem.getTag() ^ BERTag.CONTEXT) == 1) {
                target.setGraceLoginsRemaining(intValue.getValue());
            }
        }

        // error
        if ((tag.getTag() ^ BERTag.CONTEXT) == 1) {
            BERSequence sequence = (BERSequence) tag.getValue();
            BEREnumerated enum = (BEREnumerated) sequence.elementAt(0);
            target.setErrorCode(enum.getValue());
        }
    }
}