package dw.samples.ldapcontrols;

import javax.naming.ldap.Control;
import javax.naming.ldap.ControlFactory;

import com.ibm.asn1.ASN1Exception;
import com.ibm.asn1.ASN1Tag;
import com.ibm.asn1.BERDecoder;

/**
 * Transforms a control object to a PasswordPolicyResponseControl object, if
 * appropriate. Note that IBM BER library shipped with IBM Tivoli Directory
 * Server ("IBMLDAPJavaBer.jar") is necessary to compile and run this example.
 * The library is currently not supported for external use.
 * 
 * developerWorks, Take advantage of IBM Tivoli Directory Server's LDAP Controls
 * via Java and JNDI.
 * 
 * @author Stefan Zoerner
 */
public class PasswordPolicyControlFactoryIbmBer extends ControlFactory {

    /**
     * Constant representing a warning.
     */
    private static final int PWD_WARNING_TAG = 0;

    /**
     * Constant representing an error.
     */
    private static final int PWD_ERROR_TAG = 1;

    /**
     * Constant representing the time before expiration
     */
    private static final int TIME_EXP_TAG = 0;

    /**
     * Constant representing the grace logins remaining
     */
    private static final int GRACE_LOGINS_TAG = 1;

    /**
     * Creates an instance of PasswordPolicyResponseControl if the passed
     * control is a response control of this type. Attributes of the result are
     * filled with the correct values (e.g. error code).
     * 
     * @param ctl
     *            the control the check
     * @return a response control of type PasswordPolicyResponseControl, or null
     */
    public Control getControlInstance(Control ctl) {
        Control result = null;

        if (ctl.getID().equals(PasswordPolicyControl.OID)) {
            try {
                PasswordPolicyResponseControl rctl = new PasswordPolicyResponseControl();
                if (ctl.getEncodedValue() != null) {

                    BERDecoder decoder = new BERDecoder(ctl.getEncodedValue());
                    int seq = decoder.decodeSequence();

                    int tags[] = { PWD_WARNING_TAG, PWD_ERROR_TAG };
                    int tag = decoder.decodeChoice(tags);
                    int tagNum = ASN1Tag.getTagNumber(tag);
                    tag = decoder.decodeExplicit(tag);

                    if (tagNum == PWD_WARNING_TAG) {
                        handleWarning(decoder, rctl);
                    }

                    if (tagNum == PWD_ERROR_TAG) {
                        rctl.setErrorCode(decoder.decodeEnumeration());
                    }

                    decoder.endOf(seq);
                }
                result = rctl;
            } catch (ASN1Exception e) {
                e.printStackTrace();
            }
        }
        return result;
    }

    protected void handleWarning(BERDecoder decoder, PasswordPolicyResponseControl target)
            throws ASN1Exception {

        int tags[] = { GRACE_LOGINS_TAG, TIME_EXP_TAG };
        int tag = decoder.decodeChoice(tags);
        int tagNum = ASN1Tag.getTagNumber(tag);
        tag = decoder.decodeExplicit(tag);

        if (tagNum == GRACE_LOGINS_TAG) {
            target.setGraceLoginsRemaining(decoder.decodeIntegerAsInt());
        }
        if (tagNum == TIME_EXP_TAG) {
            target.setTimeBeforeExpiration(decoder.decodeIntegerAsInt());
        }
    }
}