package dw.samples.ldapcontrols;

import javax.naming.Binding;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.OperationNotSupportedException;
import javax.naming.ldap.Control;
import javax.naming.ldap.InitialLdapContext;
import javax.naming.ldap.LdapContext;

/**
 * Sample use of the Tree Delete Control with fallback.
 * 
 * Main program to delete an entry and its children with the help of the
 * TreeDeleteControl class, or recursivly if the control is not supported
 * by the target LDAP server.
 * 
 * developerWorks, Take advantage of IBM Tivoli Directory Server's LDAP Controls
 * via Java and JNDI, Listing 5: Use of the Tree Delete Control with fallback.
 * 
 * @author Stefan Zoerner
 */
public class PortableDeletion {

    public static void main(String[] args) throws NamingException {

        LdapContext ctx = new InitialLdapContext();

        LdapContext entry = (LdapContext) ctx.lookup("ou=subtree");
        entry.setRequestControls(new Control[] { new TreeDeleteControl() });
        try {
            entry.unbind("");
            System.out.println("Entry " + entry.getNameInNamespace() + " deleted");
        } catch (OperationNotSupportedException e) {
            entry.setRequestControls(new Control[0]);
            deleteRecursively(entry);
        }
    }

    public static void deleteRecursively(LdapContext entry) throws NamingException {
        NamingEnumeration enum = entry.listBindings("");
        while (enum.hasMore()) {
            Binding b = (Binding) enum.next();
            if (b.getObject() instanceof LdapContext) {
                deleteRecursively((LdapContext) b.getObject());
            }
        }
        entry.unbind("");
        System.out.println("Entry " + entry.getNameInNamespace() + " deleted");
    }
}