package dw.samples.ldapcontrols;

import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.DirContext;
import javax.naming.directory.ModificationItem;
import javax.naming.ldap.Control;
import javax.naming.ldap.InitialLdapContext;
import javax.naming.ldap.LdapContext;

/**
 * Sample use of the Password Policy Control.
 * 
 * Simple main program that changes the password of a user several times to
 * demonstrate password policy features.
 * 
 * developerWorks, Take advantage of IBM Tivoli Directory Server's LDAP Controls
 * via Java and JNDI, Listing 11: Sample code that uses the Control class.
 * 
 * @author Stefan Zoerner
 */
public class TestSetPassword {

    public static final String USER_DN = "cn=Stefan Zoerner,dc=example,dc=com";

    public static final String USER_PWD = "Start001";

    public static void main(String[] args) throws NamingException {

        Control[] rctls = { new PasswordPolicyControl(true) };

        Hashtable env = new Hashtable();
        env.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory");
        env.put(Context.PROVIDER_URL, "ldap://magritte:389/dc=example,dc=com");
        env.put(Context.SECURITY_PRINCIPAL, USER_DN);
        env.put(Context.SECURITY_CREDENTIALS, USER_PWD);

        env.put(LdapContext.CONTROL_FACTORIES,
                "dw.samples.ldapcontrols.PasswordPolicyControlFactoryIbmBer");

        System.out.println("logon on with DN " + USER_DN);
        LdapContext ctx = new InitialLdapContext(env, rctls);
        handleResponseControls(ctx);

        String[] passwords = new String[] { 
                "My1Secret", 
                "abc001",    // too short
                "01234567",  // numbers only
                USER_PWD     // current password

        };

        ctx.setRequestControls(rctls);
        for (int i = 0; i < passwords.length; i++) {
            ModificationItem replacePw = new ModificationItem(
                    DirContext.REPLACE_ATTRIBUTE, new BasicAttribute("userPassword",
                            passwords[i]));

            try {
                System.out.println("Try to set password to [" + passwords[i] + "]");
                ctx.modifyAttributes("cn=Stefan Zoerner",
                        new ModificationItem[] { replacePw });
            } catch (NamingException ne) {
                System.out.println(ne);
            }
            handleResponseControls(ctx);
        }

    }

    public static void handleResponseControls(LdapContext ctx) throws NamingException {
        Control[] ctls = ctx.getResponseControls();
        for (int j = 0; ctls != null && j < ctls.length; j++) {
            System.out.println(ctls[j]);
        }
    }
}