package dw.samples.ldapcontrols.test;

import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.ldap.Control;
import javax.naming.ldap.InitialLdapContext;
import javax.naming.ldap.LdapContext;

import dw.samples.ldapcontrols.PasswordPolicyControl;

/**
 * Bind to an LDAP server anonymously, then logon to get response control.
 * 
 * developerWorks, Take advantage of IBM Tivoli Directory Server's LDAP Controls
 * via Java and JNDI.
 * 
 * @author Stefan Zoerner
 */
public class FailedBindWithControls {

    static final String USER_DN = "cn=Test User,dc=example,dc=com";

    static final String USER_START_PWD = "Start001";

    public static void main(String[] args) throws NamingException {

        Hashtable env = new Hashtable();
        env.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory");
        env.put(Context.PROVIDER_URL, "ldap://magritte:389/");
        env.put(LdapContext.CONTROL_FACTORIES,
                "dw.samples.ldapcontrols.PasswordPolicyControlFactoryIbmBer");

        // 1. bind anonymously
        InitialLdapContext ctx = new InitialLdapContext(env, null);

        // 2. set request control to elicit pwd policy control
        Control[] rctls = { new PasswordPolicyControl(true) };
        ctx.setRequestControls(rctls);

        // 3. bind on same context with credentials
        try {
            ctx.addToEnvironment(Context.SECURITY_AUTHENTICATION, "simple");
            ctx.addToEnvironment(Context.SECURITY_PRINCIPAL, USER_DN);
            ctx.addToEnvironment(Context.SECURITY_CREDENTIALS, USER_START_PWD);
            ctx.lookup("");
        } catch (NamingException ne) {
            System.err.println(ne);
        }

        // 4. fetch response control
        Control[] responseControls = ctx.getResponseControls();
        for (int i = 0; responseControls != null && i < responseControls.length; i++) {
            System.out.println(responseControls[i]);
        }
    }
}